type op = Add | Sub | Mult | Div | Equal | Neq | Less | Leq | Greater | Geq | And | Or

type variable = 
		IntVar of string
	| ArrayVar of string * int
  | SymAssign of string * string
	| EqAssign of string * string
	| PolyAssign of string * string
	| TermVar of string

type fparam =
		Int of string
	| Array of string
	| Sym of string
	| Eq of string
	| Poly of string
	| Term of string

type varcall = 
		IntVarCall of string
	| ArrayVarCall of string * expr
	| SymVarCall of string
	| EqVarCall of string
	| PolyVarCall of string
	| TermVarCall of string
  | DeRefId of varcall * string
  | DeRefArray of varcall * string
and expr =
    Literal of int
  | StringLiteral of string
  | Var of varcall
  | Binop of expr * op * expr
  | Assign of varcall * expr
  | ExprAssign of expr * expr
  | Call of string * expr list
  | LocalCall of expr * expr list
  | Noexpr

type functiontype =
	VoidRet
  | IntRet
  | ArrayRet

type stmt =
    Block of stmt list
  | Expr of expr
  | Return of expr
  | If of expr * stmt * stmt
  | For of expr * expr * expr * stmt
  | While of expr * stmt
	| Print of expr

type functiondecl = {
		ftype : functiontype;
    fname : string;
    formals : fparam list;
    locals : variable list;
    body : stmt list;
  }

type program = functiondecl list
