open Ast

let string_of_variable = function
		ArrayVar(s, i) -> "int[] " ^ s ^ "= new int[" ^ string_of_int i ^ "];\n" 
	| IntVar(s) -> "int " ^ s ^ ";\n"
	| SymAssign(s, t) -> "Sym " ^ s ^ "= new Sym(\"" ^ t ^ "\");\n"
	| EqAssign(s, t) -> "Eq " ^ s ^ "= new Eq(\"" ^ t ^ "\");\n"
	| PolyAssign(s, t) -> "Poly " ^ s ^ "= new Poly(\"" ^ t ^"\");\n"
	| TermVar(s) -> "Term " ^ s ^ ";\n"

let rec string_of_variablecall = function
	  IntVarCall(s) -> s
	| SymVarCall(s) -> s
	| EqVarCall(s) -> s
	| PolyVarCall(s) -> s
	| TermVarCall(s) -> s
	| ArrayVarCall(s, e) -> s ^ "[" ^ string_of_expr e ^ "]"
	| DeRefId(s, a) -> string_of_variablecall s ^ "." ^ a
	| DeRefArray(s, a) -> string_of_variablecall s ^ "[" ^ a ^ "]"
and string_of_expr = function
    Literal(l) -> string_of_int l
  | StringLiteral(l) -> "\"" ^ l ^ "\""
 	| Var(s) -> string_of_variablecall s
  | Binop(e1, o, e2) ->
      string_of_expr e1 ^ " " ^
      (match o with
	Add -> "+" | Sub -> "-" | Mult -> "*" | Div -> "/" 
      | Equal -> "==" | Neq -> "!=" | And -> "&&" | Or -> "||"  
      | Less -> "<" | Leq -> "<=" | Greater -> ">" | Geq -> ">=") ^ " " ^
      string_of_expr e2
  | Assign(v, e) -> string_of_variablecall v ^ " = " ^ string_of_expr e
  | ExprAssign(e1, e2) -> string_of_expr e1 ^ " = " ^ string_of_expr e2
  | Call(f, el) -> 		
		 "MEL." ^ f ^ "(" ^ String.concat ", " (List.map string_of_expr el) ^ ")"
  | LocalCall(e, el) -> string_of_expr e ^ "(" ^ String.concat ", " (List.map string_of_expr el) ^ ")"
  | Noexpr -> ""

let rec string_of_stmt = function
    Block(stmts) ->
      "{\n" ^ String.concat "" (List.map string_of_stmt stmts) ^ "}\n"
  | Expr(expr) -> string_of_expr expr ^ ";\n";
  | Return(expr) -> "return " ^ string_of_expr expr ^ ";\n";
  | If(e, s, Block([])) -> "if (" ^ string_of_expr e ^ ")\n" ^ string_of_stmt s
  | If(e, s1, s2) ->  "if (" ^ string_of_expr e ^ ")\n" ^
      string_of_stmt s1 ^ "else\n" ^ string_of_stmt s2
  | For(e1, e2, e3, s) ->
      "for (int " ^ string_of_expr e1  ^ " ; " ^ string_of_expr e2 ^ " ; " ^
      string_of_expr e3  ^ ") " ^ string_of_stmt s
  | While(e, s) -> "while (" ^ string_of_expr e ^ ") " ^ string_of_stmt s
	| Print(e) -> "System.out.println(" ^ string_of_expr e ^ ");\n"

let string_of_fparam = function
    Int(id) -> "int " ^ id
  | Sym(id) -> "Sym " ^ id
  | Eq(id) -> "Eq " ^ id
  | Poly(id) -> "Poly " ^ id
  | Term(id) -> "Term " ^ id
  | Array(id) -> "int[] "^ id

let string_of_functiontype = function
		VoidRet -> "void"
	| IntRet -> "int"
	| ArrayRet -> "int[]" 	

let string_of_fdecl fdecl =
  "public static " ^ string_of_functiontype fdecl.ftype ^ " " ^
	fdecl.fname ^ "(" ^ String.concat ", " (List.map string_of_fparam fdecl.formals) ^ ")\n{\n" ^
  String.concat "" (List.map string_of_variable fdecl.locals) ^
  String.concat "" (List.map string_of_stmt fdecl.body) ^
  "}\n"

let string_of_program funcs =
	"public class MEL { \n" ^
  String.concat "\n" (List.map string_of_fdecl funcs) ^ "\n" ^
	"public static void main(String args[]){"^
	"MEL.evaluate();"^
	"}"^
	"} \n"
