%{ open Ast %}

%token SEMI LPAREN RPAREN LBRACE RBRACE LBRACKET RBRACKET COMMA DOT COLON
%token PLUS MINUS TIMES DIVIDE MOD POWER ASSIGN
%token EQ NEQ LT LEQ GT GEQ
%token AND OR
%token RETURN IF ELSE FOR WHILE INT VOID ARRAY SYM EQN POLY TERM FUNC PRINT
%token <int> LITERAL
%token <string> STRINGLITERAL
%token <string> ID
%token EOF

%nonassoc NOELSE
%nonassoc ELSE
%right ASSIGN
%left DOT
%left EQ NEQ
%left LT GT LEQ GEQ
%left PLUS MINUS
%left TIMES DIVIDE MOD POWER
%left SQRT SUM AVG ABS LOG

%start program
%type <Ast.program> program

%%

program:
   /* nothing */ { [] }
 | program functiondecl { $2 :: $1 }

functiondecl:
   functype ID COLON formals_opt LBRACE variabledecl_list stmt_list RBRACE
     { { ftype = $1;
			fname = $2;
	 formals = $4;
	 locals = List.rev $6;
	 body = List.rev $7 } }

functype:
		FUNC VOID  { VoidRet }
	| FUNC INT { IntRet }
	| FUNC ARRAY { ArrayRet }

formals_opt:
    /* nothing */ { [] }
  | formal_list   { List.rev $1 }

formaldecl:
		INT ID               { Int($2) }
	| ARRAY ID             { Array($2) }
	| SYM ID               { Sym($2) }
	| EQN ID               { Eq($2) }
	| POLY ID              { Poly($2) }
	| TERM ID              { Term($2) }

formal_list:
    formaldecl                        { [$1] }
  | formal_list COMMA formaldecl   { $3 :: $1 }

variabledecl_list:
    /* nothing */    { [] }
  | variabledecl_list variabledecl { $2 :: $1 }

variabledecl:
    INT ID SEMI { IntVar($2) }
  | ARRAY ID LPAREN LITERAL RPAREN SEMI { ArrayVar($2, $4) } 
  | SYM ID ASSIGN STRINGLITERAL SEMI { SymAssign($2, $4) }
  | EQN ID ASSIGN STRINGLITERAL SEMI { EqAssign($2, $4) }
  | POLY ID ASSIGN STRINGLITERAL SEMI { PolyAssign($2, $4) }
  | TERM ID SEMI { TermVar($2) }

variablecall:
    ID                                 { IntVarCall($1) }
  | ID LPAREN expr RPAREN              { ArrayVarCall($1, $3) } 
  | variablecall DOT ID                { DeRefId($1, $3) } 
  | variablecall LBRACKET ID RBRACKET  { DeRefArray($1, $3) }

stmt_list:
    /* nothing */  { [] }
  | stmt_list stmt { $2 :: $1 }

stmt:
    expr SEMI { Expr($1) }
  | RETURN expr SEMI { Return($2) }
  | LBRACE stmt_list RBRACE { Block(List.rev $2) }
  | IF LBRACKET expr_if RBRACKET stmt %prec NOELSE { If($3, $5, Block([])) }
  | IF LBRACKET expr_if RBRACKET stmt ELSE stmt    { If($3, $5, $7) }
  | FOR LBRACKET expr_opt SEMI expr_opt SEMI expr_opt RBRACKET stmt
     { For($3, $5, $7, $9) }
  | WHILE LBRACKET expr RBRACKET stmt { While($3, $5) }
  | PRINT expr SEMI { Print($2) }

expr_opt:
    /* nothing */ { Noexpr }
  | expr          { $1 }

expr_if:
    expr          { $1 }
  | expr AND expr { Binop($1, And,   $3) }
  | expr OR  expr { Binop($1, Or,   $3) }

expr:
    LITERAL          { Literal($1) }
  | STRINGLITERAL    { StringLiteral($1) }
  | expr PLUS   expr { Binop($1, Add,   $3) }
  | expr MINUS  expr { Binop($1, Sub,   $3) }
  | expr TIMES  expr { Binop($1, Mult,  $3) }
  | expr DIVIDE expr { Binop($1, Div,   $3) }
  | expr EQ     expr { Binop($1, Equal, $3) }
  | expr NEQ    expr { Binop($1, Neq,   $3) }
  | expr LT     expr { Binop($1, Less,  $3) }
  | expr LEQ    expr { Binop($1, Leq,   $3) }
  | expr GT     expr { Binop($1, Greater,  $3) }
  | expr GEQ    expr { Binop($1, Geq,   $3) }
  | variablecall     { Var($1) }
  | expr ASSIGN expr { ExprAssign($1, $3) }
  | LBRACKET expr COLON actuals_opt RBRACKET { LocalCall($2, $4) }
  | LBRACKET expr RBRACKET { $2 }

actuals_opt:
    /* nothing */ { [] }
  | actuals_list  { List.rev $1 }

actuals_list:
    expr                    { [$1] }
  | actuals_list COMMA expr { $3 :: $1 }

